@@ -15,6 +15,9 @@
 --     muc_room_locking = false
 --     muc_room_default_public_jids = true
 --
+
+module:depends('room_destroy');
+
 -- we use async to detect Prosody 0.10 and earlier
 local have_async = pcall(require, 'util.async');
 
@@ -26,15 +29,18 @@ end
 local jid_node = require 'util.jid'.node;
 local jid_host = require 'util.jid'.host;
 local jid_split = require 'util.jid'.split;
-local json = require 'util.json';
+local json = require 'cjson.safe';
 local st = require 'util.stanza';
 local uuid_gen = require 'util.uuid'.generate;
 
 local util = module:require 'util';
 local internal_room_jid_match_rewrite = util.internal_room_jid_match_rewrite;
 local is_healthcheck_room = util.is_healthcheck_room;
+local process_host_module = util.process_host_module;
 
 local BREAKOUT_ROOMS_IDENTITY_TYPE = 'breakout_rooms';
+-- Available breakout room functionality
+local RENAME_FEATURE = 'http://jitsi.org/protocol/breakout_rooms#rename';
 -- only send at most this often updates on breakout rooms to avoid flooding.
 local BROADCAST_ROOMS_INTERVAL = .3;
 -- close conference after this amount of seconds if all leave.
@@ -42,6 +48,7 @@ local ROOMS_TTL_IF_ALL_LEFT = 5;
 local JSON_TYPE_ADD_BREAKOUT_ROOM = 'features/breakout-rooms/add';
 local JSON_TYPE_MOVE_TO_ROOM_REQUEST = 'features/breakout-rooms/move-to-room';
 local JSON_TYPE_REMOVE_BREAKOUT_ROOM = 'features/breakout-rooms/remove';
+local JSON_TYPE_RENAME_BREAKOUT_ROOM = 'features/breakout-rooms/rename';
 local JSON_TYPE_UPDATE_BREAKOUT_ROOMS = 'features/breakout-rooms/update';
 
 local main_muc_component_config = module:get_option_string('main_muc');
@@ -159,13 +166,18 @@ function broadcast_breakout_rooms(room_jid)
             end
         end
 
-        local json_msg = json.encode({
+        local json_msg, error = json.encode({
             type = BREAKOUT_ROOMS_IDENTITY_TYPE,
             event = JSON_TYPE_UPDATE_BREAKOUT_ROOMS,
             roomCounter = main_room._data.breakout_rooms_counter,
             rooms = rooms
         });
 
+        if not json_msg then
+            module:log('error', 'not broadcasting breakout room information room:%s error:%s', main_room_jid, error);
+            return;
+        end
+
         for _, occupant in main_room:each_occupant() do
             if jid_node(occupant.jid) ~= 'focus' then
                 send_json_msg(occupant.jid, json_msg)
@@ -232,6 +244,28 @@ function destroy_breakout_room(room_jid, message)
 end
 
 
+function rename_breakout_room(room_jid, name)
+    local main_room, main_room_jid = get_main_room(room_jid);
+
+    if room_jid == main_room_jid then
+        return;
+    end
+
+    if main_room then
+        if main_room._data.breakout_rooms then
+            main_room._data.breakout_rooms[room_jid] = name;
+            local breakout_room = breakout_rooms_muc_service.get_room_from_jid(room_jid);
+
+            if breakout_room then
+                breakout_room:set_subject(breakout_room.jid, name);
+            end
+
+        end
+        main_room:save(true);
+        broadcast_breakout_rooms(main_room_jid);
+    end
+end
+
 -- Handling events
 
 function on_message(event)
@@ -256,7 +290,7 @@ function on_message(event)
     local room = get_room_by_name_and_subdomain(session.jitsi_web_query_room, session.jitsi_web_query_prefix);
 
     if not room then
-        module:log('warn', 'No room found found for %s/%s',
+        module:log('warn', 'No room found for %s/%s',
                 session.jitsi_web_query_prefix, session.jitsi_web_query_room);
         return false;
     end
@@ -277,13 +311,13 @@ function on_message(event)
             end
         end
         if not occupant then
-            log('warn', 'No occupant %s found for %s', from, room.jid);
+            module:log('warn', 'No occupant %s found for %s', from, room.jid);
             return false;
         end
     end
 
     if occupant.role ~= 'moderator' then
-        log('warn', 'Occupant %s is not moderator and not allowed this operation for %s', from, room.jid);
+        module:log('warn', 'Occupant %s is not moderator and not allowed this operation for %s', from, room.jid);
         return false;
     end
 
@@ -293,16 +327,23 @@ function on_message(event)
     elseif message.attr.type == JSON_TYPE_REMOVE_BREAKOUT_ROOM then
         destroy_breakout_room(message.attr.breakoutRoomJid);
         return true;
+    elseif message.attr.type == JSON_TYPE_RENAME_BREAKOUT_ROOM then
+        rename_breakout_room(message.attr.breakoutRoomJid, message.attr.subject);
+        return true;
     elseif message.attr.type == JSON_TYPE_MOVE_TO_ROOM_REQUEST then
         local participant_jid = message.attr.participantJid;
         local target_room_jid = message.attr.roomJid;
 
-        local json_msg = json.encode({
+        local json_msg, error = json.encode({
             type = BREAKOUT_ROOMS_IDENTITY_TYPE,
             event = JSON_TYPE_MOVE_TO_ROOM_REQUEST,
             roomJid = target_room_jid
         });
 
+        if not json_msg then
+            module:log('error', 'skip sending request room:%s error:%s', room.jid, error);
+        end
+
         send_json_msg(participant_jid, json_msg)
         return true;
     end
@@ -318,8 +359,7 @@ function on_breakout_room_pre_create(event)
     -- Only allow existent breakout rooms to be started.
     -- Authorisation of breakout rooms is done by their random uuid name
     if main_room and main_room._data.breakout_rooms and main_room._data.breakout_rooms[breakout_room.jid] then
-        breakout_room._data.subject = main_room._data.breakout_rooms[breakout_room.jid];
-        breakout_room.save();
+        breakout_room:set_subject(breakout_room.jid, main_room._data.breakout_rooms[breakout_room.jid]);
     else
         module:log('debug', 'Invalid breakout room %s will not be created.', breakout_room.jid);
         breakout_room:destroy(main_room_jid, 'Breakout room is invalid.');
@@ -334,11 +374,11 @@ function on_occupant_joined(event)
         return;
     end
 
-    local main_room = get_main_room(room.jid);
+    local main_room, main_room_jid = get_main_room(room.jid);
 
     if main_room and main_room._data.breakout_rooms_active then
         if jid_node(event.occupant.jid) ~= 'focus' then
-            broadcast_breakout_rooms(room.jid);
+            broadcast_breakout_rooms(main_room_jid);
         end
 
         -- Prevent closing all rooms if a participant has joined (see on_occupant_left).
@@ -383,14 +423,14 @@ function on_occupant_left(event)
         return;
     end
 
-    local main_room = get_main_room(room_jid);
+    local main_room, main_room_jid = get_main_room(room_jid);
 
     if not main_room then
         return;
     end
 
     if main_room._data.breakout_rooms_active and jid_node(event.occupant.jid) ~= 'focus' then
-        broadcast_breakout_rooms(room_jid);
+        broadcast_breakout_rooms(main_room_jid);
     end
 
     -- Close the conference if all left for good.
@@ -400,14 +440,39 @@ function on_occupant_left(event)
             -- and we will have the old instance
             local main_room, main_room_jid = get_main_room(room_jid);
             if main_room and main_room.close_timer then
-                module:log('info', 'Closing conference %s as all left for good.', main_room_jid);
-                main_room:set_persistent(false);
-                main_room:destroy(nil, 'All occupants left.');
+                prosody.events.fire_event("maybe-destroy-room", {
+                    room = main_room;
+                    reason = 'All occupants left.';
+                    caller = module:get_name();
+                });
             end
         end);
     end
 end
 
+-- Stop other modules from destroying room if breakout rooms not empty
+function handle_maybe_destroy_main_room(event)
+    local main_room = event.room;
+    local caller = event.caller;
+
+    if caller == module:get_name() then
+        -- we were the one that requested the deletion. Do not override.
+        return nil; -- stop room destruction
+    end
+
+    -- deletion was requested by another module. Check for break room occupants.
+    for breakout_room_jid, _ in pairs(main_room._data.breakout_rooms or {}) do
+        local breakout_room = breakout_rooms_muc_service.get_room_from_jid(breakout_room_jid);
+        if breakout_room and breakout_room:has_occupant() then
+            module:log('info', 'Suppressing room destroy. Breakout room still occupied %s', breakout_room_jid);
+            return true; -- stop room destruction
+        end
+    end
+end
+
+module:hook_global("maybe-destroy-room", handle_maybe_destroy_main_room)
+
+
 function on_main_room_destroyed(event)
     local main_room = event.room;
 
@@ -423,25 +488,6 @@ end
 
 -- Module operations
 
--- process a host module directly if loaded or hooks to wait for its load
-function process_host_module(name, callback)
-    local function process_host(host)
-        if host == name then
-            callback(module:context(host), host);
-        end
-    end
-
-    if prosody.hosts[name] == nil then
-        module:log('debug', 'No host/component found, will wait for it: %s', name)
-
-        -- when a host or component is added
-        prosody.events.add_handler('host-activated', process_host);
-    else
-        process_host(name);
-    end
-end
-
-
 -- operates on already loaded breakout rooms muc module
 function process_breakout_rooms_muc_loaded(breakout_rooms_muc, host_module)
     module:log('debug', 'Breakout rooms muc loaded');
@@ -449,6 +495,15 @@ function process_breakout_rooms_muc_loaded(breakout_rooms_muc, host_module)
     -- Advertise the breakout rooms component so clients can pick up the address and use it
     module:add_identity('component', BREAKOUT_ROOMS_IDENTITY_TYPE, breakout_rooms_muc_component_config);
 
+    -- Tag the disco#info response with available features of breakout rooms.
+    host_module:hook('host-disco-info-node', function (event)
+        local session, reply, node = event.origin, event.reply, event.node;
+        if node == BREAKOUT_ROOMS_IDENTITY_TYPE and session.jitsi_web_query_room then
+            reply:tag('feature', { var = RENAME_FEATURE }):up();
+        end
+        event.exists = true;
+    end);
+
     breakout_rooms_muc_service = breakout_rooms_muc;
     module:log("info", "Hook to muc events on %s", breakout_rooms_muc_component_config);
     host_module:hook('message/host', on_message);
