@@ -11,7 +11,7 @@ function filter_transcription_tag(event)
     local stanza = event.stanza;
     local session = event.origin;
     if stanza and stanza.name == "presence" then
-        if not is_feature_allowed(session,'transcription') then
+        if not is_feature_allowed(session.jitsi_meet_context_features,'transcription') then
             stanza:maptags(function(tag)
                 if tag and tag.name == filtered_tag_name then
                     module:log("info", "Removing %s tag from presence stanza!", filtered_tag_name);
@@ -28,4 +28,4 @@ module:hook("presence/bare", filter_transcription_tag);
 module:hook("presence/full", filter_transcription_tag);
 module:hook("presence/host", filter_transcription_tag);
 
-module:log("info", "Loaded mod_muc_transcription_filter!");
\ No newline at end of file
+module:log("info", "Loaded mod_muc_transcription_filter!");
